#ifndef PEBBLISP_H
#define PEBBLISP_H

#include "object.h"
#include "env.h"

#ifndef STANDALONE
#include <pebble.h>
#define printd(...) copySlice(NULL, NULL)
#endif

#ifdef DEBUG
#define printd(...) printf(__VA_ARGS__)
#else
#define printd(...) copySlice(NULL, NULL)
#endif

struct Slice {
    const char *text;
    char length;
};

typedef struct Result {
    Object obj;
    struct Slice *slices;
} Result;

Object eval(const Object *obj, struct Environment *env);
Result parse(struct Slice *slices);
Result readSeq(struct Slice *slices);
Object parseAtom(struct Slice *slice);
void copySlice(char * dest, struct Slice *src);
Object parseEval(const char *input, struct Environment *env);
void eval_forms(Object *destList, const Object *src, struct Environment *env);

Object evalLambdaArgs(const Object *arg_forms);

struct Environment defaultEnv();
void deleteEnv(struct Environment *e);
void addToEnv(struct Environment *env, const char *name, const Object obj);
Object fetchFromEnvironment(const char *name, struct Environment *env);
void printEnv(struct Environment *env);
struct Environment envForLambda(const Object *params, const Object *arg_forms,
                                struct Environment *outer);

Result result(Object obj, struct Slice *slices);

// Slices
void copySlice(char * dest, struct Slice *src);
void debugSlice(struct Slice *s);

#define BASIC_OP(_name) \
Object _name(Object obj1, Object obj2, struct Environment *env);
BASIC_OP(add); BASIC_OP(sub);
BASIC_OP(mul); BASIC_OP(dvi);
BASIC_OP(mod); BASIC_OP(equ);
BASIC_OP(gth); BASIC_OP(lth);
#undef BASIC_OP
Object catObjects(const Object obj1, const Object obj2, struct Environment *env);

#endif
